/*requires Schema.SQLGeneration.sql*/
/*requires Table.SQLGeneration.ExecutionLog.sql*/

DECLARE @ProcedureSchema NVARCHAR(256);
DECLARE @ProcedureName   NVARCHAR(256);

SET @ProcedureSchema = 'SQLGeneration' ;
SET @ProcedureName = 'LogNewExecution' ;

PRINT '-----------------------------------------------------------------------------------------------------------------';
PRINT 'PROCEDURE [' + @ProcedureSchema + '].[' + @ProcedureName +  ']';

IF  NOT EXISTS (SELECT 1 FROM sys.objects WHERE object_id = OBJECT_ID(N'[' + @ProcedureSchema + N'].[' + @ProcedureName +  N']') AND type in (N'P'))
BEGIN
    BEGIN TRY
        EXECUTE ('CREATE Procedure [' + @ProcedureSchema + '].[' + @ProcedureName +  '] ( ' +
                ' @ServerName    varchar(512), ' +
                ' @DbName    varchar(50) ' +
                ') ' +
                'AS ' +
                'BEGIN ' +
                '   SELECT ''Not implemented'' ' +
                'END')
    END TRY
    BEGIN CATCH
        PRINT '   Error while trying to create procedure'
        RETURN
    END CATCH

    PRINT '   PROCEDURE created.'
END
GO

ALTER PROCEDURE [SQLGeneration].[LogNewExecution] (
    @PurposeName                    VARCHAR(512),
    @StartTime                      DATETIME,
    @NbDaysOfValidity               INT, 
    @GenerationId                   BIGINT OUTPUT,
    @Debug                          BIT = 0
)
AS
/*
  ===================================================================================
    DESCRIPTION:

    PARAMETERS:

    REQUIREMENTS:

    EXAMPLE USAGE :
    
        EXEC [SQLGeneration].[LogNewExecution] @Debug = 1
  ===================================================================================
*/
BEGIN
    SET NOCOUNT ON;
    DECLARE @tsql               nvarchar(max);
    DECLARE @LineFeed           CHAR(2);

    SELECT
        @tsql               = '',
        @LineFeed           = CHAR(13) + CHAR(10)
    ;

    if (@Debug = 1)
    BEGIN
        PRINT '-- -----------------------------------------------------------------------------------------------------------------';
        PRINT '-- Now running [SQLGeneration].[LogNewExecution] stored procedure.';
        PRINT '-- -----------------------------------------------------------------------------------------------------------------';
    END;

    INSERT into [SQLGeneration].[ExecutionLog] (
        PurposeName,SQLLogin,ClientNetAddress,ClientHostName,ClientProgramName,StartTime,ValidForNbDays
    )
    SELECT
        @PurposeName,
        s.login_name,
        c.client_net_address,
        s.[host_name],
        s.program_name,
        @StartTime,
        @NbDaysOfValidity
    from sys.dm_exec_requests r
    inner join sys.dm_exec_sessions s
    on r.session_id = s.session_id
    left join sys.dm_exec_connections c
    on r.session_id = c.session_id
    WHERE r.session_id = @@spid ;
    
    SET @GenerationId = @@IDENTITY;

    if (@Debug = 1)
    BEGIN
        PRINT '-- -----------------------------------------------------------------------------------------------------------------';    
        PRINT '-- Execution of [SQLGeneration].[LogNewExecution] completed.';
        PRINT '-- -----------------------------------------------------------------------------------------------------------------';
    END;
END
GO


IF (@@ERROR = 0)
BEGIN
    PRINT '   PROCEDURE altered.';
END
ELSE
BEGIN
    PRINT '   Error while trying to alter procedure';
    RETURN
END;
GO

PRINT '-----------------------------------------------------------------------------------------------------------------';
PRINT '';
GO

